#import "DirectoryBrowserController.h"
#import "DirectoryBrowserItem.h"
#import "DirectoryBrowserCell.h"
#import "ImageBrowserController.h"


@implementation DirectoryBrowserController

-(void)awakeFromNib
{
	// initialize the root items (we use the home directory and the Volumes directory)
	mRootDirectoryItems = [[NSMutableArray alloc] init];
	DirectoryBrowserItem * home = [[DirectoryBrowserItem alloc] initWithPath:NSHomeDirectory() andParent:nil];
	DirectoryBrowserItem * volumes = [[DirectoryBrowserItem alloc] initWithPath:@"/Volumes/" andParent:nil];
	[mRootDirectoryItems addObject:home];
	[mRootDirectoryItems addObject:volumes];
	[mDirectoryBrowserView reloadData];
	[home release];
	[volumes release];
	
	// initialize the directory browser cell
	mDirectoryCell = [[DirectoryBrowserCell alloc] init];
	[mDirectoryCell setEditable:NO];
}

-(void)dealloc
{
	[mRootDirectoryItems release];
	[mDirectoryCell release];
	[super dealloc];
}

#pragma mark NSOutlineViewDataSource implementation

-(id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item
{
	if (item == nil)	return [mRootDirectoryItems objectAtIndex:index];
	else				return [[item children] objectAtIndex:index];
}

-(BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	if (item == nil || [[item children] count] > 0)
		return YES;
	return NO;
}

-(NSInteger)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	if (item == nil)
		return [mRootDirectoryItems count];
	return [[item children] count];
}

-(id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	if (item == nil)
		return @"/";
	
	// we use the tableColumn identifier as the key to retrieve the item's value.
	// This way, if we add other columns, we simply have to add the equivalent value
	// in the DirectoryBrowserItem class without having to modify this method.
	return [item valueForKey:[tableColumn identifier]];
}

-(void)outlineViewSelectionDidChange:(NSNotification *)notification
{
	NSInteger selectedRow = [mDirectoryBrowserView selectedRow];
	DirectoryBrowserItem * item = [mDirectoryBrowserView itemAtRow:selectedRow];
	[mImageBrowserController setPath:[item path]];
}

#pragma mark custom cells rendering

-(NSCell *)outlineView:(NSOutlineView *)outlineView dataCellForTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	// we don't want to return a unique cell for the whole row, so when tableColumn
	// is nil, return nil. This method will then be called again, this time with the
	// different columns
	if (tableColumn == nil)
		return nil;
	
	// check the column identifier to return our custom cell if needed
	if ([[tableColumn identifier] isEqualToString:@"name"])
		return mDirectoryCell;
	
	// default behavior
	return [tableColumn dataCell];
}

-(void)outlineView:(NSOutlineView *)outlineView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	if ((tableColumn == nil) || [[tableColumn identifier] isEqualToString:@"name"])
	{
		// We know that the cell at this column is our image and text cell, so grab it
		DirectoryBrowserCell * customCell = (DirectoryBrowserCell *)cell;
		
		// Set the image here since the value returned from outlineView:objectValueForTableColumn:... didn't specify the image part...
		DirectoryBrowserItem * itemObject = (DirectoryBrowserItem *)item;
		[customCell setImage:[[[NSWorkspace sharedWorkspace] iconForFile:[itemObject path]] retain]];
		[customCell setStringValue:[itemObject name]];
	}
}


@end
